﻿using FLY.OBJComponents.Client;
using FLY.Weight.Client;
using FLY.Weight.Common;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace FLY.Weight.UI.Client
{
    /// <summary>
    /// Page_Flow.xaml 的交互逻辑
    /// </summary>
    public partial class Page_MixTable : Page
    {
        
        WeightSystemClient mWeighterCsService;
        WeighterC mWeighter;

        BufferServiceClient<FlyData_Mix> mBufferServiceClient;
        BufferWindow<FlyData_Mix> mWindow;


        public Page_MixTable()
        {
            InitializeComponent();

        }

        public void Init(WeighterC w)
        {
            mWeighter = w;
            templateRoot.DataContext = w;
            mWeighterCsService = TDGage.Current.mWeighterCsService;
            SysParam sysparam = SysParam.Current;

            
            int item_idx = mWeighterCsService.Items.IndexOf(w);
            


            mBufferServiceClient = new BufferServiceClient<FlyData_Mix>(FLY.Weight.OBJ_INTERFACE.OBJ_INTERFACE.WEIGHTS_OBJ_MIX_ID(item_idx));
            mWindow = new BufferWindow<FlyData_Mix>(mBufferServiceClient, SysParam.Current.WindowSize);
            Misc.BindingOperations.SetBinding(SysParam.Current, "WindowSize", mWindow, "Size", Misc.BindingOperations.BindingMode.TwoWay);

            FObjBase.FObjSys.Current.Connect_to_Another_OBJSys(
                sysparam.ServerEP,
                mBufferServiceClient.ID);

            //界面........................................

            InitDataGrid();

            grid_window.DataContext = mWindow;
 
        }

        void InitDataGrid()
        {
            DataGrid datagrid = gridFlows;
            for (int i = 0; i < mWeighter.Ingredients.Count(); i++)
            {
                // kg
                DataGridTextColumn dgtc = new DataGridTextColumn()
                {
                    Binding = new Binding() { Path = new PropertyPath($"Items[{i}].Disp"), StringFormat = "{0:F4}" }
                };
                StackPanel sp = new StackPanel() { Orientation = Orientation.Horizontal };
                sp.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldHeader") as Style,
                        Text = $"{mWeighter.Ingredients[i].Number}#"
                    });
                sp.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldContent_mm") as Style,
                        Text = "kg",
                        Margin = new Thickness(10, 0, 10, 0)
                    });
                dgtc.Header = sp;
                datagrid.Columns.Add(dgtc);

                // %
                DataGridTextColumn dgtc2 = new DataGridTextColumn()
                {
                    Binding = new Binding() { Path = new PropertyPath($"Items[{i}].PDisp"), StringFormat = "{0:F1}" }
                };
                StackPanel sp2 = new StackPanel() { Orientation = Orientation.Horizontal };
                sp2.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldHeader") as Style,
                        Text = $"{mWeighter.Ingredients[i].Number}#"
                    });
                sp2.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldContent_mm") as Style,
                        Text = "%",
                        Margin = new Thickness(10, 0, 10, 0)
                    });
                dgtc2.Header = sp2;
                datagrid.Columns.Add(dgtc2);

            }
        }

        private void button_back_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.GoBack();
            mBufferServiceClient.Dispose();
            mWindow.Dispose();
        }

        private void button_clear_click(object sender, RoutedEventArgs e)
        {
            if (FLY.ControlLibrary.MyMessageBox.Show("确定是否清空全部历史数据?") == true)
            {
                mBufferServiceClient.Reset();
                
                FLY.ControlLibrary.Window_Tip.Show("成功", "清除完毕!", TimeSpan.FromSeconds(2));
            }
        }

        private void button_prepage_Click(object sender, RoutedEventArgs e)
        {
            mWindow.MovePrePage();
        }

        private void button_nextpage_Click(object sender, RoutedEventArgs e)
        {
            mWindow.MoveNextPage();
        }

        private void button_newest_Click(object sender, RoutedEventArgs e)
        {
            mWindow.MoveNewest();
        }
    }
}
