﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using OfficeOpenXml;
using System.ComponentModel;
using System.Text.RegularExpressions;

namespace Autogen_WS
{
    public class DataInfos:INotifyPropertyChanged
    {
       
        List<WS_ItemInfo> infos = new List<WS_ItemInfo>();

        DeviceAddr mDeviceAddrs = new DeviceAddr();
        DeviceTranslationTable mDeviceTranslationTable = new DeviceTranslationTable();


        public double Progress { get; set; }
        public string XlsxFile { get; set; }
        public bool GenWeighterC { get; set; }
        public bool GenWS { get; set; }

        /// <summary>
        /// WeighterC.cs
        /// </summary>
        public string CSharpPath_Weighter { get; set; }

        /// <summary>
        /// WeighterAccessory.cs
        /// </summary>
        public string CSharpPath_Accessory { get; set; }

        /// <summary>
        /// 主界面的dispatcher
        /// </summary>
        public Action<string> ErrorMsgbox;

        public DataInfos()
        {

        }
        //public DataInfos(string xmlfile)
        //{
        //    LoadXmlFile(xmlfile);
        //}



        /// <summary>
        /// 分析 "设备1_读写4WUB3821" 分拆为plc序号，寄存器类型, 寄存器地址
        /// </summary>
        /// <param name="plc_addr">寄存器地址字符串</param>
        /// <param name="plc_index">plc序号</param>
        /// <param name="type">寄存器类型</param>
        /// <param name="addr">寄存器地址</param>
        /// <returns>是否成功</returns>
        bool AnalysisAddr(string plc_addr, out int plc_index, out string type, out int addr)
        {
            plc_index = -1;
            type = null;
            addr = 0;
            //string plc_addr = "设备1_读写4WUB3821";
            //Console.WriteLine("查找对象=" + plc_addr);

            //查表
            var cells = from _cell in mDeviceTranslationTable.Tables where _cell.PLCVarName == plc_addr select _cell;
            if (cells.Count() == 0)
                return false;

            type = cells.First().MBVarType;
            addr = cells.First().MBAddr;
            plc_index = cells.First().PLCIndex;


            return true;

        }


        /// <summary>
        /// 加载 HMI-53435最后确定.xlsm
        /// </summary>
        /// <param name="xlsxfile"></param>
        /// <returns></returns>
        public bool LoadXlsxFile()
        {
            string xlsxfile = XlsxFile;
            //加载地址
            if (!mDeviceAddrs.Load(xlsxfile))
            {
                ErrorMsgbox?.Invoke("地址表加载异常");
                return false;
            }
            //加载转换表
            if (!mDeviceTranslationTable.Load(xlsxfile, mDeviceAddrs.EP.Count()))
            {
                ErrorMsgbox?.Invoke("加载转换表异常");
                return false;
            }

            infos.Clear();
            Progress = 0;

            if (!LoadXlsxFile_Weighter(xlsxfile))
            {
                ErrorMsgbox?.Invoke("加载 各层参数 异常");
                return false;
            }

            if (!LoadXlsxFile_Accessory(xlsxfile))
            {
                ErrorMsgbox?.Invoke("加载 参数 异常");
                return false;
            }
            return true;
        }
        /// <summary>
        /// 各层参数
        /// </summary>
        /// <param name="xlsxfile"></param>
        /// <returns></returns>
        public bool LoadXlsxFile_Weighter(string xlsxfile)
        {
            //加载地址
            if (!mDeviceAddrs.Load(xlsxfile))
            {
                ErrorMsgbox?.Invoke("地址表加载异常");
                return false;
            }
            //加载转换表
            if (!mDeviceTranslationTable.Load(xlsxfile, mDeviceAddrs.EP.Count()))
            {
                ErrorMsgbox?.Invoke("加载转换表异常");
                return false;
            }

            int i_propDescription = 2;//描述
            int i_propName = 3;//属性名
            int i_typeName = 4;//类型
            int i_scale = 5;//放大
            int i_canWrite = 6;//可写
            int i_plc_addr = 7;//PLC寄存器

            int weighter_cnt = 0;//


            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["各层参数"])
                {
                    int i = i_plc_addr;

                    Regex r_header_addr = new Regex(@"[A-F]层HMI寄存器");

                    while (true)
                    {
                        string str;
                        try
                        {
                            str = sheet.Cells[1, i].Value.ToString();
                        }
                        catch
                        {
                            break;
                        }
                        if (str == null) break;
                        if (str == "") break;
 
                        if (r_header_addr.IsMatch(str))
                        {
                            weighter_cnt++;
                        }
                        i++;
                        continue;
                    }
                    if (weighter_cnt <= 1)
                    {
                        ErrorMsgbox?.Invoke("X 层HMI寄存器 找不到");
                        return false;
                    }

                    // 生成C#对象属性列表

                    i = 2;
                    while (true)
                    {
                        WS_ItemInfo ii = new WS_ItemInfo();

                        try
                        {
                            ii.propDescription = sheet.Cells[i, i_propDescription].Value.ToString();
                        }
                        catch
                        {
                            //没有描述，肯定结束了
                            break;
                        }


                        try
                        {
                            ii.propName = sheet.Cells[i, i_propName].Value.ToString();
                        }
                        catch
                        {
                            ErrorMsgbox?.Invoke("没有属性名 描述:"+ ii.propDescription);
                            return false;
                            
                        }
                        
                        try
                        {
                            ii.typeName = sheet.Cells[i, i_typeName].Value.ToString();
                        }
                        catch
                        {
                            ErrorMsgbox?.Invoke("没有类型异常 描述:" + ii.propDescription);
                            return false;
                        }


                        try
                        {
                            string scale_str = sheet.Cells[i, i_scale].Value.ToString();
                            if (!string.IsNullOrEmpty(scale_str))
                            {
                                double d;
                                if (double.TryParse(scale_str, out d))
                                {
                                    ii.scale = d;
                                }
                            }
                        }
                        catch
                        {

                        }

                        ii.propName = ii.propName.Trim();
                        ii.typeName = ii.typeName.Trim();
                        ii.propDescription = ii.propDescription.Trim();

                        //propertyname 不能有重复
                        for (int wi = 0; wi < weighter_cnt; wi++)
                        {
                            ii.objectName = "Items[" + wi.ToString() + "]";

                            if (infos.Any((info) => { return (info.propName == ii.propName) && (info.objectName == ii.objectName); }))
                            {
                                ErrorMsgbox?.Invoke("相同的属性名 描述:" + ii.propDescription);
                                return false;
                            }

                            string plc_addr;
                            try
                            {
                                plc_addr = sheet.Cells[i, i_plc_addr + wi].Value.ToString();
                            }
                            catch
                            {
                                //空的
                                continue;
                            }

                            if (!AnalysisAddr(plc_addr, out ii.plc_index, out ii.reg_type, out ii.mb_addr))
                            {
                                ErrorMsgbox?.Invoke("不能转换为modbus地址 描述:" + ii.propDescription);
                                return false;
                            }

                            infos.Add(ii);
                            ii = ii.Clone();
                        }
                        i++;
                        Progress += 1;
                        if (Progress >= 100)
                            Progress = 50;
                    }

                }
            }
            Progress = 100;
            return true;
        }

        /// <summary>
        /// 参数
        /// </summary>
        /// <param name="xlsxfile"></param>
        /// <returns></returns>
        public bool LoadXlsxFile_Accessory(string xlsxfile)
        {
            int i_propDescription = 2;//描述
            int i_propName = 3;//属性名
            int i_typeName = 4;//类型
            int i_scale = 5;//放大
            int i_canWrite = 6;//可写
            int i_plc_addr = 7;//PLC寄存器

            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["参数"])
                {
                    
                    // 生成C#对象属性列表
                    int i = 2;
                    while (true)
                    {
                        WS_ItemInfo ii = new WS_ItemInfo();

                        try
                        {
                            ii.propDescription = sheet.Cells[i, i_propDescription].Value.ToString();
                        }
                        catch
                        {
                            //没有描述，肯定结束了
                            break;
                        }


                        try
                        {
                            ii.propName = sheet.Cells[i, i_propName].Value.ToString();
                        }
                        catch
                        {
                            ErrorMsgbox?.Invoke("没有属性名 描述:" + ii.propDescription);
                            return false;

                        }

                        try
                        {
                            ii.typeName = sheet.Cells[i, i_typeName].Value.ToString();
                        }
                        catch
                        {
                            ErrorMsgbox?.Invoke("没有类型异常 描述:" + ii.propDescription);
                            return false;
                        }


                        try
                        {
                            string scale_str = sheet.Cells[i, i_scale].Value.ToString();
                            if (!string.IsNullOrEmpty(scale_str))
                            {
                                double d;
                                if (double.TryParse(scale_str, out d))
                                {
                                    ii.scale = d;
                                }
                            }
                        }
                        catch
                        {

                        }

                        ii.propName = ii.propName.Trim();
                        ii.typeName = ii.typeName.Trim();
                        ii.propDescription = ii.propDescription.Trim();

                        //propertyname 不能有重复

                        ii.objectName = "Accessory";

                        if (infos.Any((info) => { return (info.propName == ii.propName) && (info.objectName == ii.objectName); }))
                        {
                            ErrorMsgbox?.Invoke("异常:相同的属性名 描述:" + ii.propDescription);
                            return false;
                        }

                        string plc_addr;
                        try
                        {
                            plc_addr = sheet.Cells[i, i_plc_addr].Value.ToString();
                        }
                        catch
                        {
                            ErrorMsgbox?.Invoke("异常:没有地址 描述:" + ii.propDescription);
                            return false;
                        }

                        if (!AnalysisAddr(plc_addr, out ii.plc_index, out ii.reg_type, out ii.mb_addr))
                        {
                            ErrorMsgbox?.Invoke("异常:不能转换为modbus地址 描述:" + ii.propDescription);
                            return false;
                        }

                        infos.Add(ii);
                        
                        i++;
                        Progress += 1;
                        if (Progress >= 100)
                            Progress = 50;
                    }
                    Progress = 100;
                    return true;
                }
            }
        }


        void Gen_CS_Core(StreamWriter sw, Dictionary<string, WS_ItemInfo> d)
        {
            sw.WriteLine("        #region 辅助代码生成");
            foreach (KeyValuePair<string, WS_ItemInfo> sii in d)
            {
                WS_ItemInfo info = sii.Value;
                sw.WriteLine("        /// <summary>");
                sw.WriteLine("        /// {0}", info.propDescription);
                sw.WriteLine("        /// </summary>");
                sw.Write("        public {0} {1}", info.GetPropertyType(), info.propName);
                sw.WriteLine(" { get; set; }");
                sw.WriteLine();
            }
            sw.WriteLine("        #endregion");
        }

        void Gen_CS_WeighterAccessory(StreamWriter sw)
        {

            Dictionary<string, WS_ItemInfo> d = new Dictionary<string, WS_ItemInfo>();

            foreach (WS_ItemInfo ii in infos)
            {
                if (ii.objectName != "Accessory") continue;
                if (d.ContainsKey(ii.propName)) continue;
                d.Add(ii.propName, ii);
            }
            Gen_CS_Core(sw, d);

        }

        void Gen_CS_WeighterItem(StreamWriter sw)
        {

            Dictionary<string, WS_ItemInfo> d = new Dictionary<string, WS_ItemInfo>();
            
            Regex regex_objectname = new Regex(@"Items\[[0-9]\]");
            foreach (WS_ItemInfo ii in infos)
            {
                if(!regex_objectname.IsMatch(ii.objectName))
                    continue;

                if (d.ContainsKey(ii.propName))
                    continue;

                d.Add(ii.propName, ii);
            }

            Gen_CS_Core(sw, d);
        }

        void Gen_CS(Action<StreamWriter> core, string objname, string csharppath)
        {
            //找到 #region 辅助代码生成 与最接近的 #endregion

            //StringBuilder sb = new StringBuilder();
            List<String> file_content_header = new List<string>();
            List<String> file_content_tailer = new List<string>();
            int state = 0;
            int region_cnt = 0;

            string weightc_new_path = Path.Combine(Path.GetDirectoryName(XlsxFile), "Generated", objname + ".cs");

            if (!Directory.Exists(Path.GetDirectoryName(weightc_new_path)))
                Directory.CreateDirectory(Path.GetDirectoryName(weightc_new_path));

            StreamWriter sw = new StreamWriter(weightc_new_path, false, Encoding.UTF8);

            if (File.Exists(csharppath))
            {
                StreamReader sr = new StreamReader(csharppath);

                //把整个文件读到 sb中
                while (!sr.EndOfStream)
                {
                    string line = sr.ReadLine();
                    switch (state)
                    {
                        case 0:
                            if (line.Contains("#region 辅助代码生成"))
                            {
                                //找到了
                                state = 1;
                                region_cnt = 1;
                            }
                            else
                            {
                                sw.WriteLine(line);
                            }
                            break;
                        case 1://跳过中间的
                            if (line.Contains("#region"))
                            {
                                region_cnt++;
                            }
                            else if (line.Contains("#endregion"))
                            {
                                region_cnt--;
                                if (region_cnt == 0)
                                {
                                    //插入 自动代码
                                    core(sw);

                                    //下面都是尾
                                    state = 2;
                                }
                            }
                            break;

                        case 2://记录尾巴
                            sw.WriteLine(line);
                            break;
                    }
                }

                sr.Close();
                sw.Flush();
                sw.Close();
            }
            else
            {
                sw.WriteLine("#region 辅助代码生成");
                core(sw);
                sw.WriteLine("#endregion");
                sw.Flush();
                sw.Close();
            }


        }

        public void Gen_CS()
        {
            Gen_CS(Gen_CS_WeighterAccessory, "WeighterAccessory", CSharpPath_Accessory);
            Gen_CS(Gen_CS_WeighterItem, "WeighterC", CSharpPath_Weighter);
        }

        public void Gen_WS_XML()
        {
            PLCGroup plcgroup = new PLCGroup();

            for (int i = 0; i < mDeviceAddrs.EP.Count(); i++)
            {
                string ep_str = mDeviceAddrs.EP[i];
                plcgroup.Devices.Add(new PLCGroup.PLCDevice() { EP = Misc.StringConverter.ToIPEndPoint(ep_str) });
            }
            foreach (WS_ItemInfo ii in infos)
            {
                plcgroup.Variables.Add(new PLCGroup.PLCVariable()
                {
                    DeviceIndex = ii.plc_index,
                    Mode = ii.reg_type,
                    Addr = ii.mb_addr,
                    Type = ii.typeName,
                    Scale = ii.scale,
                    OwnerName = ii.objectName,
                    PropertyName = ii.propName
                });
            }
            string ws_path = Path.Combine(Path.GetDirectoryName(XlsxFile), "Generated", "WS.xml");
            plcgroup.Save(ws_path);
        }


        public event PropertyChangedEventHandler PropertyChanged;
    }
}
